%Indeterminacy and Forecastability
%Ippei Fujiwara and Yasuo Hirose
%Replication Code

clear all;
randn('state', 0);

%*******************************************************************
%**      assign names to parameters
%********************************************************************/
psi1 = 0.9;
%psi1 = 1.1;
psi2 = 0.25;
rhor = 0.5;
pist = 4;
rst  = 2;
kap  = 0.5;
sig  = 2;
rhog = 0.7;
rhoz = 0.7;
rhogz= 0.0;
Mr   = 0;
Mg   = 0;
Mz   = 0;
sigr = 0.31;
sigg = 0.38;
sigz = 1.00;
sigs = 0.25;
%sigs = 1.00;

bet  = 1/(1+rst/400);

%/********************************************************************
%**      matrices of canonical system
%********************************************************************/

%/** equation indices **/
eq_is     = 1;
eq_pc     = 2;
eq_rule   = 3;
eq_g      = 4;
eq_z      = 5;
eq_Ey     = 6;
eq_Epi    = 7;

%/** variable indices **/
v_y     = 1;
v_pi    = 2;
v_r     = 3;
v_g     = 4;
v_z     = 5;
v_Ey    = 6;
v_Epi   = 7;

%/** shock indices **/
e_r = 1;
e_g = 2;
e_z = 3;

%/** expectation error indices **/
n_y  = 1;
n_pi = 2;

%/** summary **/
neq  = 7;
neps = 3;
neta = 2;

%/** initialize matrices **/
GAM0 = zeros(neq,neq);
GAM1 = zeros(neq,neq);
   C = zeros(neq,1);
 PSI = zeros(neq,neps);
 PPI = zeros(neq,neta);

%/**********************************************************
%**      1. IS curve 
%**********************************************************/
GAM0(eq_is,v_y)   =  1;
GAM0(eq_is,v_Ey)  = -1;
GAM0(eq_is,v_r)   =  1/sig;
GAM0(eq_is,v_Epi) = -1/sig;
GAM0(eq_is,v_g)  = -1;

%/**********************************************************
%**      2. Phillips curve 
%**********************************************************/
GAM0(eq_pc,v_pi)  = 1;
GAM0(eq_pc,v_Epi) = -bet;
GAM0(eq_pc,v_y)   = -kap;
GAM0(eq_pc,v_z)   = kap;

%/**********************************************************
%**      3. Monetary policy rule 
%**********************************************************/
GAM0(eq_rule,v_r)  = 1;
GAM1(eq_rule,v_r)  = rhor;
GAM0(eq_rule,v_pi) = -(1-rhor)*psi1;
GAM0(eq_rule,v_y)  = -(1-rhor)*psi2;
GAM0(eq_rule,v_z)  =  (1-rhor)*psi2;
PSI(eq_rule,e_r)   = 1;

%/**********************************************************
%**      4.-5. Shock processes
%**********************************************************/
%/** g **/
GAM0(eq_g,v_g)  = 1;
GAM1(eq_g,v_g)  = rhog;
PSI(eq_g,e_g)    = 1;
%/** z **/
GAM0(eq_z,v_z)  = 1;
GAM1(eq_z,v_z)  = rhoz;
PSI(eq_z,e_z)    = 1;

%/**********************************************************
%**      6.-7. Expectation errors
%**********************************************************/
%/** E(y) **/
GAM0(eq_Ey,v_y)  = 1;
GAM1(eq_Ey,v_Ey) = 1;
 PPI(eq_Ey,n_y)  = 1;
%/** E(pi) **/
GAM0(eq_Epi,v_pi)  = 1;
GAM1(eq_Epi,v_Epi) = 1;
 PPI(eq_Epi,n_pi)  = 1;

%/********************************************************************
%**  Solve the model: QZ(generalized Schur) decomposition by GENSYS
%********************************************************************/
[G1,C,impact,fmat,fwt,ywt,gev,eu,TETA]=gensys(GAM0,GAM1,C,PSI,PPI);

%/********************************************************************
%**  Specify a particular sunspot solution under indeterminacy
%********************************************************************/
% if eu(2)==0   %/* indeterminacy */
%     %/* compute a psistar on the boundary */
%     psi1star = 1 - bet*psi2*(1/bet-1)/kap-1E-8;
%     GAM0star = GAM0;
%     GAM0star(eq_rule,v_pi) = -(1-rhor)*psi1star;
% 
%     %/* Evaluate Model under determinacy by shifting cutoff */
%     [G1star,Cstar,impactstardet,fmatstar,fwtstar,ywtstar,gevstar,eustardet,loosestar] = gensys(GAM0star,GAM1,C,PSI,PPI,1-1E-6);
% 
%     %/* Compute M matrix that guarantees continuity on the boundary */
%     META = inv(TETA'*TETA)*TETA'*(impactstardet-impact);
%     impact = impact + TETA*META;
% end

%/* Incorporate TETA (sunspot effect) into the impact matrix */
if eu(2)==1   %/* determinacy */
   RR = [impact zeros(size(impact,1),1)];
elseif eu(2)==0   %/* indeterminacy */
   RR = [impact TETA];
end

%/********************************************************************
%**  Chol decomp of Covariance Matrix with M's
%********************************************************************/
%/* Assumption: ez affects only z, while eg affects both g and z, following
%LS(2004).
omega = zeros(4,4);

omega(1,1) = sigr;
omega(2,2) = sigg;
omega(3,2) = rhogz*sigz;
omega(3,3) = sqrt(sigz^2-omega(3,2)^2);
omega(4,4) = sigs;

rotmat = [ 1 0 0 Mr;
           0 1 0 Mg;
           0 0 1 Mz;
           0 0 0 1];

sig_chol = rotmat*omega;

%/********************************************************************
%**  Generate simulated series
%********************************************************************/
nrep = 500;
nsim = 300;
ndrop =100;

smoms = zeros(nrep,3);

for i=1:nrep
    
    % block of shocks to be used in simulation
    shocks = randn(neps+1,nsim);
    
    yt = zeros(neq,1);
    yts =zeros(nsim,neq);
    
    for s=1:nsim
        yt = G1*yt+RR*sig_chol*shocks(:,s);
        yts(s,:)=yt';
    end
    
    yts(1:ndrop,:) = [];   %eliminate initial simulations
    t = nsim - ndrop;
    
    %compute variances and AR(1) coefficient
    vars  = yts(1:t,1:2).*yts(1:t,1:2);   %variances of output and inflation
    ar1y = inv(yts(1:t-1,1)'*yts(1:t-1,1))*yts(1:t-1,1)'*yts(2:t,1);   %AR(1) coef for output
    ar1p = inv(yts(1:t-1,2)'*yts(1:t-1,2))*yts(1:t-1,2)'*yts(2:t,2);   %AR(1) coef for inflation
    yf   = ar1y*yts(1:t,1);   % one-step-ahead forecast for output
    pf   = ar1p*yts(1:t,2);   % one-step-ahead forecast for inflation
    rmsey = sqrt(((yf(1:t-1,1)-yts(2:t,1))'*(yf(1:t-1,1)-yts(2:t,1)))/(t-1));
    rmsep = sqrt(((pf(1:t-1,1)-yts(2:t,2))'*(pf(1:t-1,1)-yts(2:t,2)))/(t-1));
    yf4   = (ar1y^4)*yts(1:t,1);   % four-step-ahead forecast for output
    pf4   = (ar1p^4)*yts(1:t,2);   % four-step-ahead forecast for inflation
    rmsey4 = sqrt(((yf(1:t-4,1)-yts(5:t,1))'*(yf(1:t-4,1)-yts(5:t,1)))/(t-4));
    rmsep4 = sqrt(((pf(1:t-4,1)-yts(5:t,2))'*(pf(1:t-4,1)-yts(5:t,2)))/(t-4));
    
    smoms(i,:) = [sqrt(mean(vars(:,2))) ar1p rmsep];
    
end

smom = mean(smoms);
